'use strict';
var gulp = require('gulp');
var clean = require('gulp-clean');
var less = require('gulp-less');
var replace = require('gulp-replace-path');
var fileinclude = require('gulp-file-include');
var autoprefixer = require('gulp-autoprefixer');
var through2 = require('through2');
var path = require('path');
var minifyCss = require('gulp-minify-css');


var fs = require('fs');

var ftp = require('vinyl-ftp');
var plumber = require('gulp-plumber');
var gulpWebpack = require('webpack-stream');
var webpack = require('webpack');

var named = require('vinyl-named');

var runSequence = require('run-sequence');

const image = require('gulp-image');

var config = setting;

var utils = {
    extend: function () {
        var result = {};
        if (arguments.length == 1) {
            var a = arguments[0];
            result = copy(result, a);
        }
        if (arguments.length > 1) {
            for (var j = 1, len = arguments.length; j < len; j++) {
                result = copy(result, arguments[j]);
            }
        }
        return result;
        function copy(o1, o2) {
            for (var key in o2) {
                o1[key] = o2[key];
            }
            return o1;
        }
    },
    parseJson: function (src) {
        var jsonPath = path.resolve(src + "/build.json");
        try {
            var file = fs.readFileSync(jsonPath)
            file = JSON.parse(String(file.contents));
            return file;
        } catch (err) {
            return -1;
        }

    },
    isRelativePath: function (path) {
        var result = true;
        if (path.search(/[:;#\<]|\)|\(/) > -1) {
            result = false;
        }
        return result;
    },
    reBuildGolb:function(){

    },
    replacePath: function ($0, $1, __absolutePath__, relativePath) {
        if (path.isAbsolute($1) || !utils.isRelativePath($1)) {
            return $0;
        }

        // $1 = encodezURI($1)
        if (config.env == "local") {
            var releaseUri = "";
        } else {
            var releaseUri = config.cdn_path;
        }

        var imageDir = path.dirname(__absolutePath__);
        //取到图片完整路径
        imageDir = path.resolve(imageDir, $1);
       
        // imageDir = decodeURI(imageDir);

        //获取相对于base的相对路径
        var version = new Date().toJSON().replace(/[^\d]/g, '');
        var relativeDir = path.relative(relativePath, imageDir);
        
        var finalUri = releaseUri + '/' + relativeDir + "?t=" + version;

        finalUri = finalUri.replace(/\\/g, '/');
        finalUri = $0.replace($1, finalUri);

        return finalUri;
    },
    getGitConfig:function(){
        var gitConfig = process.cwd()+"/.git/config"
        if(fs.existsSync(gitConfig)){
            var file = fs.readFileSync(path.resolve(gitConfig)).toString();
            var url = file.match(/\[remote "origin"\][\s]*?url\s*?\=\s*?(.*)\s*/i);
            if(url){
                url = url[1];
            }
            return url;
        }
    },
    connect: function () {
        if (arguments.length == 0) {
            return undefined;
        }
        if (arguments.length == 1) {
            return arguments[0];
        }
        var step = arguments[arguments.length - 1];
        var arr = [];
        for (var i = 0, len = arguments.length - 1; i < len; i++) {
            arguments[i] = arguments[i] || "";
            arr.push(arguments[i].replace(new RegExp("^" + step + "*"), ""));
        }
        return arr.join(step).replace(new RegExp(step + "{1,}$"), "");
    }

};

//文件清除
gulp.task('clean', function () {
    return gulp.src([config.dist, config.cache])
        .pipe(clean({force: true}));

});

/*******************************************less编译*********************************************************/
//复制缓存
gulp.task('copyLessCache', function () {
    return  gulp.src([config.src + '/**/*.{less,css}', "!"+config.src +"/"+(config.dist||"")+"/**/*"], {base: config.src})
        .pipe(gulp.dest(config.cache));
  
});
//缓存替换路径
gulp.task('replaceLessCache', ["copyLessCache"], function () {
    return gulp.src([config.cache + '/**/*.{less,css}'])
        .pipe(replace(/url\([\"\']{0,1}(.*?)[\"\']{0,1}\)/g, function (match, __absolutePath__) {
            function replaceMethod($0, $1, $2, $3) {


                var finalUri = utils.replacePath($0, $1, __absolutePath__, setting.cache)

                return finalUri;
            }

            return replaceMethod;
        }))

        .pipe(gulp.dest(config.cache));
 
});
//编译less，导出到目标路径
gulp.task('compileLess', ["replaceLessCache"], function () {
    if (config.specify && config.specify.css) {
        var srcPath = config.cache + "/" + config.specify.css;

    } else {
        var srcPath = config.cache;
    }
    return gulp.src([srcPath + "/**/*.{less,css}"], {base: setting.cache + "/"})
        .pipe(plumber({
            errorHandler:function(err){
                console.log(err);
                this.emit("end")
            }
        }))
        .pipe(less())
        .pipe(minifyCss())
        .pipe(autoprefixer({
            browsers: ['last 2 versions', 'Android >= 4.0'],
            cascade: true, //是否美化属性值 默认：true 
            remove: true //是否去掉不必要的前缀 默认：true
        }))
        .pipe(gulp.dest(config.dist + "/"));

});
//删除缓存less
gulp.task('delLessCache', ["compileLess"], function () {
    return gulp.src(config.cache)
        .pipe(clean());
});
//less
gulp.task("less", ["copyLessCache", "compileLess", "replaceLessCache"], function () {})


/******************************************* end*********************************************************/

/*******************************************html处理*********************************************************/
gulp.task('html', function () {
    if (config.specify) {
        var basSrc = utils.connect(config.src, config.specify.html, "/")
    } else {
        var basSrc = config.src
    }

    return gulp.src([config.src + '/**/*.{htm,html}', "!"+config.src +"/"+(config.dist||"")+"/**/*"], {base: basSrc})
        .pipe(fileinclude({
            prefix: '@@',
            basepath: '@file'
        }))
        .pipe(through2.obj(function (file, enc, cb) {
            var git = utils.getGitConfig();
            if(git){
                file.contents = new Buffer(String(file.contents).replace("<head>",'<head>\n\t<meta name="git" content="'+utils.getGitConfig()+'">'));
            }            
            cb(null,file)
        }))
        .pipe(replace(/link.*?href=[\"\']([\S^\"\']*)[\"\']|src=[\"\']([\S^\"\']*)[\"\']|url\([\"\']{0,1}(.*)[\"\']{0,1}\)|__uri\([\'\"](.*?)[\'\"]\)/g, function (match, __absolutePath__) {
            function replaceMethod($0, $1, $2, $3,$4) {
                $1 = $1 || $2 || $3 || $4;
                $1 = $1 || "#";

                var finalUri = utils.replacePath($0, $1, __absolutePath__, setting.src);
                if(finalUri.search("__uri")>-1){
                    finalUri = finalUri.replace(/(__uri\()|\)/g,"")
                }
                
                return finalUri;
            };
            return replaceMethod;
        }))
        .pipe(gulp.dest(config.dist + ''));
});
/*******************************************end*********************************************************/

/*******************************************图片处理*********************************************************/

if (config.env != "local") {
    gulp.task('img', function (cb) {
        return gulp.src([config.src + '/**/*.{jpg,png,gif,mp3,swf,webp,svg}', "!"+config.src +"/"+(config.dist||"")+"/**/*"], {base: setting.src})
            .pipe(image({
                pngquant: true,
                optipng: false,
                zopflipng: false,
                jpegRecompress: false,
                jpegoptim: true,
                mozjpeg: false,
                gifsicle: false,
                svgo: false,
                concurrent: 5
            }))
            .pipe(gulp.dest(config.dist + ""))
    })

} else {
    gulp.task('img', function (cb) {

        return gulp.src([config.src + '/**/*.{jpg,png,gif,mp3,swf,webp,svg}', "!"+config.src +"/"+(config.dist||"")+"/**/*"], {base: setting.src})
            .pipe(gulp.dest(config.dist + ""))

    })
}

/*******************************************end*********************************************************/

/*******************************************webpack*********************************************************/

gulp.task("js", function (callback) {
    var packConfig = {
        devtool: 'source-map'
    };
    if (config.env == "pro") {
        packConfig.devtool = false;
        packConfig.plugins = [
            new webpack.optimize.UglifyJsPlugin({
                compress: {
                    warnings: false
                }
            })
        ]
    }
    return gulp.src([config.src + '/**/*.js', '!**lib/**/*.js', "!"+config.src +"/"+(config.dist||"")+"/**/*"], {base: config.src})
        .pipe(plumber({
            errorHandler:function(){
                this.emit("end")
            }
        }))
       
        .pipe(named(function (file) {
            return file.relative.replace(".js", "");
         }))

        .pipe(gulpWebpack(packConfig))
        .pipe(replace(/__uri\([\'\"](.*?)[\'\"]\)/g, function (match, __absolutePath__) {
            function replaceMethod($0, $1, $2, $3) {

                var finalUri = utils.replacePath($0, $1, __absolutePath__, setting.src)
             
                finalUri = finalUri.replace(/(__uri\()|\)/g,"")
              
                return finalUri;
            }

            return replaceMethod;
        }))
        .pipe(gulp.dest(config.dist))
});
gulp.task("lib-js", function () {
  
   return gulp.src(config.src + '/'+((config.specify&&config.specify.lib)||"lib")+'/**/*', {base: config.src})
        .pipe(gulp.dest(config.dist));
});

/*******************************************end*********************************************************/

/*******************************************md5*********************************************************/
// gulp.task("md5",function(callback){
//     // var jsF = filter('**/*.js', {restore: true});
//     // var cssF = filter('**/*.css', {restore: true});
//     // var imgF = filter('**/*', {restore: true});

//     var stream = gulp.src(config.dist+'/**/*.!(html)')
//        // .pipe(imgF)
//         .pipe(md5(10,[config.dist+'/**/*.css',config.dist+'/**/*.{html,htm}'],{dirLevel:1}))

//        // .pipe(imgF.restore)
//        // .pipe(cssF)
//        // .pipe(md5(10,config.dist+'/**/*.html',{dirLevel:1}))
//        // .pipe(cssF.restore)
//       //  .pipe(jsF)
//        // .pipe(md5(10,config.dist+'/**/*.html',{dirLevel:1}))
//        // .pipe(jsF.restore)
//      return stream;
// });

/*******************************************end*********************************************************/
config.ftp.maxConnections = 10;
config.ftp.parallel = 10;


var instanceFtp = ftp.create(config.ftp || "");
gulp.task("ftp-html", function (cb) {
    if(!config.html){
        console.warn("页面发布路径未配置");
        cb();
        return;
    }
    return gulp.src(config.dist + "/**/*.{html,htm}", {base: config.dist, buffer: false})
    // .pipe(instanceFtp.newerOrDifferentSize( config.ftp.htmlPath))
        .pipe(instanceFtp.dest(config.html))
})

gulp.task('ftp', ["ftp-html"], function (cb) {
    if(!config.html){
        console.warn("资源发布路径未配置");
        cb();
        return;
    }
    return gulp.src(config.dist + "/**/*.{jpg,gif,png,jpeg,css,js,mp3,swf}", {base: config.dist, buffer: false})
    // .pipe(instanceFtp.newerOrDifferentSize( config.ftp.resourcePath))
        .pipe(instanceFtp.dest(config.res))
})


/*******************************************实时刷新*********************************************************/
var browserSync = require("browser-sync");


// 发布、自动刷新浏览器 
function browser() {

    browserSync({
        // host: 172.16.157.1,  //配置host
        port: 9000,             //端口号
        open: false,             //是否自动打开
        // 路径显示/d 开始
        startPath: "",  //起始显示路径
        timestamps: true,
        server: {
            directory: true,
            // routes: {           //路由配置
            //   '/d': "./dist/new.html"
            // },
            middleware: function (req, res, next) {   //中间件
                next();
            },
            baseDir: config.dist       //静态服务器的根目录
        },

        // 指定浏览器
        // browser: "google chrome" // 或  ["google chrome","firefox"]
        //browser: Browser_sync_config.browser,
        // 延迟刷新，默认0
        reloadDelay: 1,
        // 是否载入css修改，默认true
        injectChanges: false
    });


    gulp.watch([config.src + '/**/*',"!"+config.src +"/"+(config.dist||"")+"/**/*"],function () {
        runSequence("deploy",function(){
            browserSync.reload();
        })
    })

}

//实时ftp
//任务2 远程测试实时刷新
//任务3 远程生产实时刷新
function watchFtp() {
    gulp.watch([config.src + '/**/*', "!"+config.src +"/"+(config.dist||"")+"/**/*"],['deploy'])
}

if (setting.env == 'local') {
    //任务6 发布到本地

    gulp.task('deploy', function (cb) {

        runSequence('html', 'less', 'img', 'lib-js', 'js', cb)


    })
    // gulp.task('deploy',['html', 'less','lib-js','js','img']);
} else {
    //任务4 发布到远程测试
    //任务5 发布到远程生产
    if(setting.env == "build"){
        gulp.task('deploy', function (cb) {

             runSequence(["clean"],'html','less','img','lib-js','js',cb)

        })
    }else{
        gulp.task('deploy', function (cb) {

             runSequence(["clean"],'html','less','img','lib-js','js','ftp',cb)

        })
    }
    
}



/*******************************************end*********************************************************/

if (setting.env == "local") {
    gulp.task("default", function (cb) {
        runSequence("clean", "deploy",browser);
    })
} else {
    if (config.live) {
        gulp.task('default', ["deploy"], watchFtp);
    } else {
        gulp.task('default', ["deploy"]);
    }
}